function CleanUI_InitBagLock()
    CleanUIContainerLock = CleanUIContainerLock or {};

    CleanUI_ContainerFrame_HookOnClick();
    CleanUI_ContainerFrame_UpdateAll();

    hooksecurefunc("ContainerFrame_UpdateAll", CleanUI_ContainerFrame_UpdateAll);

    -- TODO:   add tooltip to lock
end

function CleanUI_ContainerFrame_HookOnClick()
    for id = 0, 4, 1 do
        container = _G["ContainerFrame"..(id + 1)];

        local itemButton;
        if (container) then
            numberOfSlots = GetContainerNumSlots(id);

            for slot=1, numberOfSlots, 1 do
                itemButton = _G[container:GetName().."Item"..slot]
                itemButton:HookScript("OnClick", CleanUI_ContainerFrameItemButton_OnClick);
            end
        end
    end
end

function CleanUI_ContainerFrame_UpdateAll()
    local numberOfSlots, itemLink;
    
    for id = 0, 4, 1 do
        container = _G["ContainerFrame"..(id + 1)];

        if (container) then
            numberOfSlots = GetContainerNumSlots(id);

            for slot=1, numberOfSlots, 1 do
                itemLink = GetContainerItemLink(id, numberOfSlots - slot + 1);

                if (itemLink) then
                    CleanUI_UpdateItemLock(_G[container:GetName().."Item"..slot], itemLink);
                end
            end
        end
    end
end

function CleanUI_ContainerFrameItemButton_OnClick(self, button)
    if (IsAltKeyDown()) then
        if (button == "LeftButton") then

            -- item lock

            local itemLink = GetContainerItemLink(self:GetParent():GetID(), self:GetID());

            if (itemLink) then
                -- lock state can't be removed by modified click
                if (not CleanUI_IsLockedItem(itemLink)) then
                    CleanUI_AddLockedItem(itemLink);
                end
            end

            ContainerFrame_UpdateAll();
        end
    end
end

function CleanUI_UpdateItemLock(itemButton, itemLink)
    if (not itemLink) then
        CleanUI_UnLockItem(itemButton)
        return;
    end

    if (CleanUI_IsLockedItem(itemLink)) then
        CleanUI_LockItem(itemButton);
    else
        CleanUI_UnLockItem(itemButton);
    end
end

function CleanUI_IsLockedItem(link)
    local itemName = GetItemInfo(link);
    local foundItemName;

    for index, foundLink in ipairs(CleanUIContainerLock) do
        foundItemName = GetItemInfo(foundLink);

        if (foundItemName == itemName) then
            return true;
        end
    end

    return false;
end

function CleanUI_AddLockedItem(link)
    tinsert(CleanUIContainerLock, link);
end

function CleanUI_RemoveLockedItem(link)
    local itemName = GetItemInfo(link);
    local foundItemName;

    for index, foundLink in ipairs(CleanUIContainerLock) do
        foundItemName = GetItemInfo(foundLink);

        if (foundItemName == itemName) then
            tremove(CleanUIContainerLock, index);

            ContainerFrame_UpdateAll();
            return true;
        end
    end
end



function CleanUI_LockItem(itemButton)
    if (not itemButton:GetParent() or string.find(itemButton:GetParent():GetName(), "ContainerFrame") == nil) then
        return;
    end
    
    itemButton:RegisterForClicks();
    itemButton:RegisterForDrag();

    if (not itemButton.unlockButton) then
        itemButton.unlockButton = CreateFrame("Button", nil, itemButton, "UIPanelButtonTemplate");

        itemButton.unlockButton.Left:Hide();
        itemButton.unlockButton.Middle:Hide();
        itemButton.unlockButton.Right:Hide();

        itemButton.unlockButton:SetSize(16, 16);
        itemButton.unlockButton:SetNormalTexture("Interface\\PETBATTLES\\PetBattle-LockIcon");
        itemButton.unlockButton:GetNormalTexture():SetTexCoord(0, 1, 0, 1);
        itemButton.unlockButton:SetPoint("TOPLEFT", itemButton, "TOPLEFT", -1, 1);

        itemButton.unlockButton:SetScript("OnClick", CleanUI_UnLockClicked);
    end

    itemButton.unlockButton:Show();
end

function CleanUI_UnLockClicked(unlockButton)
    if (IsAltKeyDown()) then
        local itemLink = GetContainerItemLink(unlockButton:GetParent():GetParent():GetID(), unlockButton:GetParent():GetID());
        CleanUI_RemoveLockedItem(itemLink);
    end
end

function CleanUI_UnLockItem(itemButton)
    if (not itemButton:GetParent() or string.find(itemButton:GetParent():GetName(), "ContainerFrame") == nil) then
        return;
    end
    
    itemButton:RegisterForClicks("LeftButtonUp", "RightButtonUp");
    itemButton:RegisterForDrag("LeftButton");

    if (itemButton.unlockButton) then
        itemButton.unlockButton:Hide();
    end
end
